import 'dart:async';

import 'package:path/path.dart' as path;

import 'package:daily_programmer_client_dart/src/generator/challenge.dart';
import 'package:daily_programmer_client_dart/src/util/entity_name.dart';
import 'package:daily_programmer_client_dart/src/generator/generator.dart';

/// Generator for sample Daily Programing project.
class ProjectGenerator extends Generator {
  static const _templateFolder = 'project';
  static final List<String> _templateFilePaths = [
    path.join('bin', 'main.dart.mustache'),
    'test',
    '.gitignore',
    'analysis_options.yaml',
    'pubspec.yaml.mustache',
    'README.md.mustache',
  ];

  /// Author in for pubspec.yaml and README.md files.
  final String author;

  /// Project email for pubspec.yaml file.
  final String email;

  /// Project name in format abc_bcd.
  final String name;

  /// Project description for pubspec.yaml file.
  final String description;

  /// Root challenge of this project.
  final ChallengeGenerator challenge;

  /// Dry run flag set from cli
  final bool dryRun;

  /// Return an instance of [ProjectGenerator] with the context of command and
  /// send context to [ChallengeGenerator].
  factory ProjectGenerator(
      EntityName projectEntityName,
      String destinationFolder,
      EntityName challengeClassEntityName,
      String author,
      String email,
      {bool dryRun: false}) {
    destinationFolder =
        path.join(destinationFolder, projectEntityName.underscored);

    final challenge = new ChallengeGenerator(
        challengeClassEntityName, path.join(destinationFolder, 'lib'),
        dryRun: dryRun);

    return new ProjectGenerator._(author, email, projectEntityName.underscored,
        projectEntityName.spaced, challenge, destinationFolder, dryRun);
  }

  ProjectGenerator._(this.author, this.email, this.name, this.description,
      this.challenge, String destinationFolder, this.dryRun)
      : super(destinationFolder);

  // Gets a map from template file name to target file name.
  Map<String, String> _getTemplateTargetPaths() {
    final results = <String, String>{};
    for (final templatePath in _templateFilePaths) {
      results[path.join(_templateFolder, templatePath)] =
          templatePath.replaceAll('.mustache', '');
    }
    return results;
  }

  @override
  Future generate() async {
    await renderAndWriteTemplates(_getTemplateTargetPaths(), dryRun: dryRun);
    await challenge.generate();
  }
}
