import 'dart:async';
import 'dart:io';

import 'package:path/path.dart' as path;
import 'package:resource/resource.dart';

import 'package:daily_programmer_client_dart/src/util/dir_wrap.dart';
import 'package:daily_programmer_client_dart/src/util/file_writer.dart';
import 'package:daily_programmer_client_dart/src/util/path_tool.dart';
import 'package:daily_programmer_client_dart/src/util/template_file.dart';

/// An abstract class defines a template generator.
abstract class Generator {
  final String _destinationFolder;

  /// Need a destination folder for generator.
  Generator(this._destinationFolder);

  /// Renders templates and writes to target files.
  Future renderAndWriteTemplates(Map<String, String> templateTargets,
      {bool dryRun = false}) async {
    for (final template in templateTargets.keys) {
      final pathStr = path.join(_destinationFolder, templateTargets[template]);
      final tmpl = new TemplateFile(template, this);
      final tmplPath = await resolveUri(new Resource(tmpl.getTemplPath()).uri);
      switch (FileSystemEntity.typeSync(tmplPath.toFilePath())) {
        case FileSystemEntityType.FILE:
          FileWriter.writer
              .write(pathStr, await tmpl.renderString(), dryRun: dryRun);
          break;
        case FileSystemEntityType.DIRECTORY:
          DirWrap.wrap.copy(pathStr, dryRun: dryRun);
          break;
        case FileSystemEntityType.LINK:
          throw new FileSystemException('$tmplPath is a link.');
        case FileSystemEntityType.NOT_FOUND:
          throw new FileSystemException('$tmplPath is not found.');
          break;
      }
    }
  }

  /// Generates files defined for this generator.
  Future generate();
}
