import 'dart:async';
import 'dart:convert' show UTF8;

import 'package:mustache/mustache.dart';
import 'package:resource/resource.dart' show Resource;

import 'package:daily_programmer_client_dart/src/util/path_tool.dart';

/// Template file class wrapping operations on mustache template.
class TemplateFile {
  /// Template file path relative to templates/.
  final String _path;

  /// Data for this template.
  final Object _data;

  /// Template file init path and data.
  TemplateFile(this._path, this._data);

  /// Renders template file on [_path] with values from [_data].
  Future<String> renderString() async {
    final uri = getTemplPath();
    final resource = new Resource(uri);
    final content = await resource.readAsString(encoding: UTF8);
    final template = new Template(content);
    return template.renderString(_data);
  }

  /// Get absolute path template file.
  String getTemplPath() =>
      fixUri('package:daily_programmer_client_dart/src/templates/$_path');
}
