import 'package:test/test.dart';

import 'package:daily_programmer_client_dart/src/util/entity_name.dart';

void main() {
  group('Entity name', () {
    test('should produce correct formats of names', () {
      final name = new EntityName('abc_bcd_cde');
      expect(name.spaced, 'Abc Bcd Cde');
      expect(name.camelCased, 'AbcBcdCde');
      expect(name.lowerCamelCased, 'abcBcdCde');
      expect(name.dashed, 'abc-bcd-cde');
      expect(name.underscored, 'abc_bcd_cde');
    });
    test('should throw for incorrect formats', () {
      expect(() => new EntityName('Abc Bcd-Cde'), throwsArgumentError);
      expect(() => new EntityName('Abc Bcd_Cde'), throwsArgumentError);
      expect(() => new EntityName('Abc Bcd cde'), throwsArgumentError);
      expect(() => new EntityName('abc Bcd Cde'), throwsArgumentError);
      expect(() => new EntityName(' Abc Bcd Cde'), throwsArgumentError);
      expect(() => new EntityName('_abc_bcd_cde'), throwsArgumentError);
      expect(() => new EntityName('-abc-bcd-cde'), throwsArgumentError);
      expect(() => new EntityName(''), throwsArgumentError);
      expect(() => new EntityName(null), throwsArgumentError);
    });
    test('should be handle to handle different types of input', () {
      expect(new EntityName('AbcBcdCde').underscored, 'abc_bcd_cde');
      expect(new EntityName('abcBcdCde').underscored, 'abc_bcd_cde');
      expect(new EntityName('abc-bcd-cde').underscored, 'abc_bcd_cde');
      expect(new EntityName('abc_bcd_cde').underscored, 'abc_bcd_cde');
    });
  });
}
