import 'package:test/test.dart';

import 'package:daily_programmer_client_dart/src/util/path_tool.dart';

void main() {
  group('Path tool: getNormalizedPath(String)', () {
    test('should be handle different types of path', () {
      expect(getNormalizedPath('/tmp/test'), '/tmp/test');
      expect(getNormalizedPath('\\tmp\\test'), '/tmp/test');
      expect(getNormalizedPath('/tmp\\test'), '/tmp/test');
      expect(getNormalizedPath('\\tmp/test'), '/tmp/test');
      expect(getNormalizedPath(''), '.');
    });
    test('should throw for incorrect path', () {
      expect(() => getNormalizedPath(null), throwsArgumentError);
    });
  });
  group('Path tool: fixUri(String)', () {
    test('should be handle valid path', () {
      expect(fixUri('/tmp/test'), '/tmp/test');
      expect(fixUri('\\tmp\\test'), '/tmp/test');
      expect(fixUri('/tmp\\test'), '/tmp/test');
      expect(fixUri('\\tmp/test'), '/tmp/test');
      expect(fixUri(''), '');
    });
    test('should throw for incorrect path', () {
      expect(() => fixUri(null), throwsArgumentError);
    });
  });
  group('Path tool: getPackageName(String)', () {
    test('should be handle different types of package', () {
      expect(getPackageName('package:abc/bcd.dart'), 'abc');
      expect(getPackageName('package:abc123/bcd.dart'), 'abc123');
      expect(getPackageName('package:ABC/bcd.dart'), 'ABC');
      expect(getPackageName('package:a-b-c/bcd.dart'), 'a-b-c');
      expect(getPackageName('package:a_b_c/bcd.dart'), 'a_b_c');
      expect(getPackageName('dart:abc/bcd.dart'), 'abc');
      expect(getPackageName('dart:abc123/bcd.dart'), 'abc123');
      expect(getPackageName('dart:ABC/bcd.dart'), 'ABC');
      expect(getPackageName('dart:a-b-c/bcd.dart'), 'a-b-c');
      expect(getPackageName('dart:a_b_c/bcd.dart'), 'a_b_c');
    });
    test('should throw for incorrect package', () {
      expect(() => getPackageName('abc:b_c_d/cde.dart'), throwsArgumentError);
      expect(() => getPackageName(''), throwsArgumentError);
      expect(() => getPackageName(null), throwsArgumentError);
    });
  });
  group('Path tool: resolveUri(Uri)', () {
    test('should be handle path of package', () async {
      expect(
          (await resolveUri(new Uri(scheme: 'package', path: 'test/test.dart')))
              .toFilePath(),
          allOf(contains('pub.dartlang.org'), endsWith('/lib/test.dart')));
    });
    test('should throw for incorrect path of package', () async {
      expect(resolveUri(new Uri(scheme: 'package', path: 'abc/bcd.dart')),
          throwsArgumentError);
      expect(resolveUri(new Uri(scheme: 'abc', path: 'test/test.dart')),
          throwsArgumentError);
      expect(resolveUri(new Uri(scheme: 'abc', path: 'abc/bcd.dart')),
          throwsArgumentError);
      expect(resolveUri(null), throwsArgumentError);
    });
  });
}
